;============================================================================;
;                                execute.asm                                 ;

;============================================================================;
;execute                                                                     ;
;    Input     : void                                                        ;
;    Output    : void                                                        ;
;    Destroyed : ALL                                                         ;


execute\open_folder_in_window_1
         bra        switch\right

execute\open_folder_in_window_2
         bra        switch\left

execute
         tst.b      window
         beq.s      execute\open_folder_in_window_1
         cmp.b      #3,window
         beq.s      execute\open_folder_in_window_2

         bsr        save_all_screen
         bsr        wait_for_no_key
         bsr        system\interrupts_on

	 pea        execute\end(pc)	; to return there after a rts

         bsr        get_handle\current  ; d0.w = handle, d1.w = index,
                                        ; a2.l = * SYMENTRY
         btst.w     #14,10(a2)          ; local bit set ?
         beq.s      execute\not_local   ; no, execute
         cmp.w      #3,8(a2)            ; already allocated ?
         bne.s      execute\typed_local ; yes, execute
         rts                            ; no, return

execute\not_local
execute\typed_local
         move.l     a2,a3                       ; saves * SYMENTRY of file
         move.w     d0,d3                       ; saves handle of file
         move.w     d1,d4                       ; saves index of file
         bsr        get_handle\folder\current   ; d0.w = handle of folder
         move.w     d4,d1                       ; restores index of file
         jsr        filelib::gettype            ; returns d2.w = type
         move.w     d3,d0                       ; restores handle of file
         move.l     a3,a2                       ; restores * SYMENTRY of file

         clr.w      tios::kb_vars+$1C           ; may be not necessary...

         tst.w      d2                          ; test type ...
         beq        execute\asm
         cmp.w      #1,d2
         beq.s      execute\library
         cmp.w      #2,d2
         beq.s      execute\program
         cmp.w      #7,d2
         beq.s      execute\viewtext
         cmp.w      #11,d2
         beq.s      execute\viewpic

execute\viewexpr
         jmp        viewlib::showEXPR

execute\viewpic
         lea        string\pic_viewer(pc),a0    ; name of viewer
         moveq.b    #2,d1                       ; number of default viewer
         bra        open_with_viewer            ; open it

execute\viewtext
         lea        string\text_viewer(pc),a0   ; name of viewer
         clr.b      d1                          ; number of default viewer
         bra        open_with_viewer            ; open it

execute\program
         bsr        get_handle\current
         move.l     a2,-(a7)
         bsr        get_handle\folder\current
         move.l     a2,-(a7)
         pea        string\pathname(pc)
         pea        string
         jsr        tios::sprintf
         pea        string
         jsr        graphlib::clr_scr
         jsr        userlib::runprog
         lea        24(a7),a7
 	 bsr        system\interrupts_off
         bsr        uninit_mode
         bsr        unalloc_file_list
         bsr        unalloc_folder_list
         bsr        init\program
         bra        wait_for_no_key

execute\library
         lea        error\libs_are_not_exec(pc),a0
         bra.s      execute\asm\error

execute\asm
         move.w     d0,-(a7)            ; handle of file
         jsr        userlib::exec       ; run it
         addq.l     #2,a7
         tst.w      d2
         beq.s      execute\restore_screen
         cmp.w      #6,d2
         bcc.s      execute\restore_screen
         lea        error_type(pc),a0
         subq.b     #1,d2
	 and.w	    #$FF,d2
	 lsl.w	    #5,d2
	 add.w      d2,a0

execute\asm\error
         move.b     #1,error_flag
         rts

execute\restore_screen
         addq.l     #4,a7
 	 bsr        system\interrupts_off
         bsr        uninit_mode
         bsr        unalloc_file_list
         bsr        unalloc_folder_list
         bsr        init\program
         bra        wait_for_no_key

execute\end
 	 bsr        system\interrupts_off
	 bsr        restore_all_screen
	 bra        wait_for_no_key

;============================================================================;
;open_with_viewer                                                            ;
;    Input     : d0.w = file handle                                          ;
;                d1.b = default viewer number                                ;
;                a0.l = viewer name                                          ;
;                a2.l = * SYMENTRY of file                                   ;
;    Output    : void                                                        ;
;    Destroyed : d0-d7/a0-a6                                                 ;

  ;============================;
  ; default viewers :          ;
  ;                            ;
  ; d1 = 0 : none              ;
  ; d1 = 1 : viewlib::showTEXT ;
  ; d1 = 2 : viewlib::showPIC  ;
  ;============================;

open_with_viewer
         clr.l      -(a7)               ; space for saving the handles :
                                        ; it means we'll have nothing to do
                                        ; before returning
         move.l     a0,a4               ; saves viewer name
         move.b     d1,d5               ; saves default viewer number
         move.w     d0,d3               ; handle of file to view

         move.l     a2,a0               ; * SYMENTRY of file
         jsr        ziplib::iscomp      ; test if zipped
         tst.b      d0                  ; not zipped ?
         bne.s      open_with_viewer\no_extracting
                                        ; d0.b = 0, no comment during unzip
         jsr        ziplib::tempfile    ; creates temporary unzipped handle

         tst.b      d0                  ; error
         beq.s      open_with_viewer\extracting_ok
         bra        errors\not_enough_memory ; NE PAS OUBLIER !!!

open_with_viewer\extracting_ok
         move.w     d1,(a7)             ; handle of unzipped file on stack
         move.w     d1,d3               ; no error, unzipping succeeded

open_with_viewer\no_extracting
         move.l     a4,a0               ; restores viewer name
         tst.b      (a0)                ; no name entered, open with default
         beq        open_with_viewer\default

         clr.w      d1                  ; one search will be enough
         jsr        filelib::search     ; a0.l = * SYMENTRY of viewer
         tst.w      d0                  ; d0.w = handle of folder, 0 if error
         beq        open_with_viewer\not_found
         move.w     12(a0),d6           ; d6.w = handle of viewer

         DEREF      d6,a2               ; address of start of file

         cmp.l      #$200000,a2         ; test if archived
         bcs.s      open_with_viewer\not_archived

;         btst.w     #9,10(a0)           ; test if archived
;         beq.s      open_with_viewer\not_archived


open_with_viewer\archived
         move.w     (a2),d0             ; size
         addq.w     #5,d0               ; + 2 for adjusting + 3 for rounding
         clr.l      d4                  ; converts to long
         move.w     d0,d4

         move.l     d4,-(a7)            ; size to alloc
         jsr        tios::HeapAlloc
         addq.l     #4,a7

         tst.w      d0                  ; no mem ?
         beq        errors\not_enough_memory ;**** NE PAS OUBLIER

         move.w     d0,2(a7)            ; handle of viewer saved on stack

         move.w     d0,d6               ; saves handle in d0
         DEREF      d0,a1               ; start address of handle
         move.l     a1,a5               ; saves address of handle in a5.l
         lsr.w      #2,d4               ; rounds to next longword boundary
         subq.w     #1,d4               ; minus one for dbf
open_with_viewer\archived\copy_loop
         move.l     (a2)+,(a1)+         ; copy...
         dbf.s      d4,open_with_viewer\archived\copy_loop
         move.l     a5,a2               ; address of viewer

open_with_viewer\not_archived
         move.w     14(a2),d0           ; offset to _main
         cmp.w      #$6002,2(a2,d0.w)   ; is it a "bra.s _main+2" ?
         bne.s      open_with_viewer\not_modified
         move.w     d3,4(a2,d0.w)       ; handle put of file to view
open_with_viewer\not_modified           ; just launch (don't put handle)
         move.w     d6,-(a7)            ; push handle of viewer
         jsr        userlib::exec       ; run it
	 addq.l     #2,a7               ; restore stack
         bra.s      open_with_viewer\exit

open_with_viewer\default
         move.w     d3,d0               ; handle of file to view
         tst.b      d5                  ; (no text viewer at this time)
         beq.s      open_with_viewer\none
         cmp.b      #1,d5               ; not used at this time
         beq.s      open_with_viewer\text_viewer

open_with_viewer\pic_viewer
	 jsr        viewlib::showPIC
         bra.s      open_with_viewer\exit

open_with_viewer\text_viewer

open_with_viewer\not_found
         lea        error\no_viewer(pc),a0
         move.b     #1,error_flag

open_with_viewer\none                   ; no viewer (if text_viewer = '0')

open_with_viewer\exit
         move.l     a0,a5
         tst.w      (a7)
         beq.s      open_with_viewer\exit\file_not_unzipped
         jsr        tios::HeapFree
open_with_viewer\exit\file_not_unzipped
         addq.l     #2,a7
         tst.w      (a7)
         beq.s      open_with_viewer\exit\viewer_not_archived
         jsr        tios::HeapFree
open_with_viewer\exit\viewer_not_archived
         addq.l     #2,a7
         move.l     a5,a0
         rts

;                                execute.asm                                 ;
;============================================================================;