;============================================================================;
;                                filelib.asm                                 ;
;============================================================================;


;============================================================================;
;search                                                                      ;
;                                                                            ;
;    Description : Allocates enough RAM to fill a handle for the folder      ;
;    Input       : a0.l = null terminated name of the file to search for     ;
;    Output      : d0.l = HSYM of file                                       ;
;    Destroyed   : d0.l/d1.w-d6.w/a0.l-a1.l/a5.l-a6.l                        ;
search
         move.w     #tios::FolderListHandle,d0   ; SYM_TABLE of folders
         DEREF      d0,a6
         move.l     (a6),d6                     ; number of folders in d6.w
         subq.w     #1,d6                       ; for dbra
         lea        16(a6),a6                   ; points to first handle

         move.l     a0,-(a7)                    ; string to search for
         subq.l     #4,a7                       ; space for second address

folders_loop
         move.w     (a6),d0                     ; folders SYM_TABLE handle
         DEREF      d0,a5                       ; folders SYM_TABLE address
         move.l     (a5)+,d5                    ; number of files in d5.w
         subq.w     #1,d5                       ; for dbra

         bmi        next_folder_loop            ; if no files

files_loop
         move.l     8(a5),d4                    ; saves last character and
         clr.l      8(a5)                       ; clears it
         move.l     a5,(a7)                     ; name of file address
         jsr        tios::strcmp                ; compares
         move.l     d4,8(a5)                    ; restores last character
         tst.w      d0
         beq.s      found_a_file

         lea        14(a5),a5                   ; next name of file
         dbra.s     d5,files_loop

next_folder_loop
         lea        14(a6),a6                   ; next handle of folder
         dbra.s     d6,folders_loop
nothing_found
         addq.l     #8,a7                       ; free space
         suba.l     a0,a0                       ; clears a0 and d0 to return
         clr.w      d0                          ; an error
         rts

found_a_file
         addq.l     #8,a7                       ; free space
;         move.l     a5,a0                       ; SYM_ENTRY address of file
         move.w     (a6),d0                     ; handle of folder
         swap       d0
         move.w     12(a5),d0                   ; handle of file
                                                ; returns HSYM value
         rts

d0_to_hex
         lea        hexbuffer(pc),a0
         moveq.w    #7,d2
d0_to_hex\loop
         moveq.b    #$F,d1
         and.b      d0,d1
         lsr.l      #4,d0
         cmp.b      #9,d1
         ble.s      one_add
         addq.b     #7,d1
one_add
         add.b      #48,d1
         move.b     d1,0(a0,d2.w)
         dbra.s     d2,d0_to_hex\loop
         rts

;============================================================================;
;gettype                                                                     ;
;                                                                            ;
;    Description : Returns the type of a file in range 0 to 14. See the      ;
;                  following comments for type values                        ;
;    Input       : d0.w = handle of folder containing the file               ;
;                  d1.w = index of file to return the type of                ;
;    Output      : d2.w = type of the file                                   ;
;    Destroyed   : ???                                                       ;
gettype

;+-------------------------------------------------------------+
;|                       Index of types                        |
;+-------------------------------------------------------------+
;+-----------+-----------+-------------------------------------+
;|   d2.w    | file type |             Description             |
;+-----------+-----------+-------------------------------------+
;|     0     |    ASM    | Assembler file on TI-89 and TI-92+  |
;+-----------+-----------+-------------------------------------+
;|     1     |    LIB    | Library                             |
;+-----------+-----------+-------------------------------------+
;|     2     |    PRGM   | Program                             |
;+-----------+-----------+-------------------------------------+
;|     3     |    FUNC   | Function or program with parameters |
;+-----------+-----------+-------------------------------------+
;|     4     |    MAT    | Matrix                              |
;+-----------+-----------+-------------------------------------+
;|     5     |    LIST   | List                                |
;+-----------+-----------+-------------------------------------+
;|     6     |    MAC    | Geometry macro                      |
;+-----------+-----------+-------------------------------------+
;|     7     |    TEXT   | Text                                |
;+-----------+-----------+-------------------------------------+
;|     8     |    STR    | String                              |
;+-----------+-----------+-------------------------------------+
;|     9     |    DATA   | Data                                |
;+-----------+-----------+-------------------------------------+
;|     10    |    FIG    | Geometry figure                     |
;+-----------+-----------+-------------------------------------+
;|     11    |    PIC    | Picture                             |
;+-----------+-----------+-------------------------------------+
;|     12    |    GDB    | Graph database                      |
;+-----------+-----------+-------------------------------------+
;|     13    |    EXPR   | Expression                          |
;+-----------+-----------+-------------------------------------+
;|     14    |    Other  | Other for the TI-89 and TI-92+      |
;+-----------+-----------+-------------------------------------+

;    Type recognition :
;
;  At the file handle address (FHA), you read (word size) the Var-Link
; displayed size - 2  = L.
;
;  The last byte seems to be at FHA+L+1. So format is :
;
;+-----------------+-------+-------+-------+-------+-------+-------+-------+
;| Offset from FHA | 0     | 1     | 2     |  ...  | L     | L + 1 | L + 2 |
;+-----------------+-------+-------+-------+-------+-------+-------+-------+
;|           Value | Hi(L) | Lo(L) |  ???  |  ???  |  $00  |  $F3  |  ???  |
;+-----------------+-------+-------+-------+-------+-------+-------+-------+
;
;  I saw following values in the tests I made for the last bytes :
;
;
;+-----------------+-------+-------+
;| Offset from FHA | L     | L + 1 |
;+-----------------+-------+-------+
;|           ASM 1 |  $00  |  $F3  |
;|               2 |  $00  |  $F3  |
;|               3 |  $00  |  $F3  |
;|               4 |  $00  |  $F3  |
;+-----------------+-------+-------+
;| printing  STR 1 |  $00  |  $2D  |
;| non-printing  2 |  $00  |  $00  | ; not tested
;+-----------------+-------+-------+
;| abc      EXPR 1 |  $63  |  $00  |
;| a             2 |  $01  |  $0B  |
;| 1             3 |  $01  |  $1F  |
;| .5            4 |  $00  |  $23  |
;| exp(15)       5 |  $01  |  $25  |
;| sqrt(-1)      6 |  $01  |  $26  |
;| 0        EXPR 7 |  $01  |  $28  |
;| a+b           8 |  $0C  |  $8B  |
;| ab-c          9 |  $0D  |  $8D  |
;| 2*a          10 |  $0B  |  $8F  |
;| ab/c         11 |  $0D  |  $91  |
;| exp(a)       12 |  $25  |  $93  |
;| I missed a lot  |  ...  |  ...  |
;+-----------------+-------+-------+
;|                 |       |       |
;
;  Notes on preceding data :
;  - don't expect L + 2 to be $00 because the next file may start right after
;    it, or other data may be there.
;  - don't use long or word size operations for reading at L + x : L may well
;    be odd, especially for strings.
;  - L isn't always zero (matrix are list of lists so this byte isn't cleared
;    but set to $D9 like L + 1) : it will be for most types. But don't
;    trust it. In expr it has no meaning for example.
;  - because of the necesary alignement of handle addresses, you'll always
;    have FHA at an even address. This also means that if the L + 1 byte is
;    at an odd address, the next one is free and seems to be cleared. This
;    happens for odd variable sizes. The FHA is not on long word boundaries.
;  - printing strings should end with $2ED and non-printing ones with $00 but
;    some EXPRs have the same terminating byte. These EXPRs are starting with
;    a $00 right after the size word.


         DEREF      d0,a0               ; folders SYMTABLE address
         and.l      #$FFFF0000,d1       ; converts to long
         lsl.l      #1,d1               ; 2 * index
         add.l      d1,a0
         lsl.l      #1,d1               ; 4 * index
         add.l      d1,a0
         lsl.l      #1,d1               ; 8 * index
         add.l      d1,a0               ; equivalent to a0 += 14 * index
         move.w     (a0),d0             ; size of file - 2
         move.b     1(a0,d0.w),d1       ; last byte of file


         rts

;============================================================================;
; data                                                                       ;
;============================================================================;

hexbuffer           dc.b      "12345678",0

;============================================================================;
;                                filelib.asm                                 ;
;============================================================================;